#include <Wire.h>
#include "SSD1306Wire.h"
#include <NTPtimeESP.h>
#include <ESP8266WiFi.h>
#define clockAddress 0x68
#define displayAddress 0x3c
#define test

unsigned int clockSecond;
unsigned int clockMinute;
unsigned int clockHour;

//general for Wemos d1 mini:
int d0 = 16;
int d1 = 5;
int d2 = 4;
int d3 = 0;
int d4 = 2;
int d5 = 14;
int d6 = 12;
int d7 = 13;
int d8 = 15;

const char* WIFI_SSID = "XXXXXXXXXX";   //Set Wifi SSID
const char* WIFI_PWD = "YYYYYYYYYY";    //Set WiFi passwd

int month;
int year;
int clockCenterX = 64;
int clockCenterY = 32;

float x1H, y1H, x2H, y2H;
float x1M, y1M, x2M, y2M;
float x3H , y3H, x4H, y4H;
float x3M , y3M, x4M, y4M;

SSD1306Wire Display(displayAddress, d2, d1);
NTPtime NTPnl("nl.pool.ntp.org"); //Set for your country
strDateTime dateTime;

void printMessage(String message) {
#ifdef test
  Serial.println(message);
#endif
}

void setup() {
  Wire.begin(d2, d1);
  pinMode(d4, INPUT_PULLUP);

#ifdef test
  Serial.begin(74880);
#endif
  String resetReason = ESP.getResetReason();
  if (resetReason != "Deep-Sleep Wake") {
    prepareDisplay();   //only do this once after cold restart!
  }
  /* Use this part only to set the time
      setDateTime();
      printMessage("Time is set");
    }
    else {  //in case of real wake up
  */
  Display.allocateBuffer();
  // Use this to resume after a deep sleep without resetting the display (what init() would do).
  // Returns true if connection to the display was established and the buffer allocated, false otherwise.
  //  }
  for (int h = 0; h < 12; h++)
  {
    drawMark(h);
  }
}

void loop() {
  getDate();
  while (clockSecond != 0) {  //synchronise at hh:mm:00
    getDate();
    delay(100);
    yield();
  }
  Display.setColor(WHITE);
  drawMin(clockMinute);
  drawHour(clockHour, clockMinute);
  ESP.deepSleep(595e5, WAKE_RF_DISABLED);
}

void getDate() {
  Wire.beginTransmission(clockAddress);
  byte zero = 0x00;
  Wire.write(zero);
  Wire.endTransmission();
  Wire.requestFrom(clockAddress, 7);
  clockSecond = bcdToDec(Wire.read());
  clockMinute = bcdToDec(Wire.read());
  clockHour = bcdToDec(Wire.read());
  month = bcdToDec(Wire.read());
  year = bcdToDec(Wire.read());
  String m = String(clockHour) + ":" + String(clockMinute) + ":" + String(clockSecond);
  printMessage(m);
}

byte bcdToDec(byte val) {
  return ((val / 16 * 10) + (val % 16));
}

void prepareDisplay() { //Do this only once after cold boot
  printMessage("prepare display");
  Display.init();
  Display.setColor(WHITE);
  Display.setFont(ArialMT_Plain_16);
  Display.setTextAlignment(TEXT_ALIGN_CENTER);
  Display.setBrightness(0);
  Display.clear();
  Display.display();
}

void drawMark(int h)
{
  int s = 28;
  int e = 28;
  if (h == 0 || h == 3 || h == 6 || h == 9) {
    e = 32;
  }
  float x1, y1, x2, y2;
  h = h * 30;
  h = h + 270;
  x1 = e * cos(h * 0.0175);
  y1 = e * sin(h * 0.0175);
  x2 = s * cos(h * 0.0175);
  y2 = s * sin(h * 0.0175);
  Display.drawLine(x1 + clockCenterX, y1 + clockCenterY, x2 + clockCenterX, y2 + clockCenterY);
}

void drawMin(int m) //Big hand!
{
  m = m * 6;
  m = m + 270;
  x1M = 25 * cos(m * 0.0175);
  y1M = 25 * sin(m * 0.0175);
  x2M = 1 * cos(m * 0.0175);
  y2M = 1 * sin(m * 0.0175);
  Display.setColor(BLACK);
  Display.drawLine(x3M + clockCenterX, y3M + clockCenterY, x4M + clockCenterX, y4M + clockCenterY);
  Display.setColor(WHITE);
  Display.drawLine(x1M + clockCenterX, y1M + clockCenterY, x2M + clockCenterX, y2M + clockCenterY);
  Display.display();
  x3M = x1M; y3M = y1M; x4M = x2M; y4M = y2M; //to erase hand next minute
}

void drawHour(int h, int m) //Small hand!
{
  h = (h * 30) + (m / 2);
  h = h + 270;
  x1H = 18 * cos(h * 0.0175);
  y1H = 18 * sin(h * 0.0175);
  x2H = 1 * cos(h * 0.0175);
  y2H = 1 * sin(h * 0.0175);
  Display.setColor(BLACK);
  Display.drawLine(x3H + clockCenterX, y3H + clockCenterY, x4H + clockCenterX, y4H + clockCenterY);
  Display.setColor(WHITE);
  Display.drawLine(x1H + clockCenterX, y1H + clockCenterY, x2H + clockCenterX, y2H + clockCenterY);
  Display.display();
  x3H = x1H; y3H = y1H; x4H = x2H; y4H = y2H; //to erase hand next minute
}

void setDateTime() {
  Display.drawString(64, 0, "Connecting");
  Display.drawString(64, 22, "to WiFi");
  delay(2000);
  WiFi.begin(WIFI_SSID, WIFI_PWD);
  int counter = 1;
  while (WiFi.status() != WL_CONNECTED) {
    yield();
    delay(500);
    Display.drawString(8 * counter, 40, ".");
    counter += 1;
    Display.display();
  }
  Display.clear();
  Display.drawString(64, 30, "Wait");
  Display.display();
  printMessage("Connected");
  Display.clear();
  Display.display();
  dateTime = NTPnl.getNTPtime(1.0, 1);
  delay(1000);
  dateTime = NTPnl.getNTPtime(1.0, 1);
  NTPnl.printDateTime(dateTime);
  byte zero = 0x00;
  Wire.beginTransmission(clockAddress);
  Wire.write(zero);
  Wire.write(decToBcd(dateTime.second));
  Wire.write(decToBcd(dateTime.minute));
  Wire.write(decToBcd(dateTime.hour));
  Wire.write(decToBcd(5));
  Wire.write(decToBcd(dateTime.day));
  Wire.write(decToBcd(dateTime.month));
  Wire.write(decToBcd(dateTime.year));
  Wire.write(zero);
  Wire.endTransmission();
}

byte decToBcd(byte val) {
  return ((val / 10 * 16) + (val % 10));
}
